﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Magify
{
    internal class CountersStorage : IDisposable
    {
        private readonly string _folder;
        private readonly Dictionary<CounterType, BinaryStorage> _storages = new();

        public CountersStorage(string folder)
        {
            _folder = folder;
        }

        public BinaryStorage this[CounterType type]
        {
            get
            {
                if (!_storages.TryGetValue(type, out var storage))
                {
                    storage = BinaryStorage
                        .Construct(Path.Combine(_folder, $"{type.ToEnumString()}.bin"))
                        .AddEntry(BinaryTypeString.Shared)
                        .AddEntry(BinaryTypeInt32.Shared)
                        .SupportDictionariesOf<string, int>()
                        .Build();
                    _storages[type] = storage;
                }
                return storage;
            }
        }

        public void Dispose()
        {
            foreach (var pair in _storages)
            {
                pair.Value.Dispose();
            }
            _storages.Clear();
        }
    }
}