using System;
using UniRx;

namespace Magify
{
    internal class SessionCounter : IInitializable, IDisposable
    {
        public static readonly TimeSpan DefaultSessionInterval = TimeSpan.FromSeconds(2);

        private readonly CompositeDisposable _disposables = new();
        private readonly GeneralPrefs _prefs;
        private readonly PlatformAPI _platformAPI;

        private DateTime _shiftToBackgroundTime = DateTime.MaxValue;

        public IReactiveProperty<TimeSpan> Interval { get; set; } = new ReactiveProperty<TimeSpan>(DefaultSessionInterval);

        public SessionCounter(GeneralPrefs prefs, PlatformAPI platformAPI)
        {
            _prefs = prefs;
            _platformAPI = platformAPI;
        }

        void IInitializable.Initialize()
        {
            StartNewSession();

            _platformAPI.OnBackground
                .Subscribe(_ => _shiftToBackgroundTime = DateTime.UtcNow)
                .AddTo(_disposables);
            _platformAPI.OnForeground
                .Where(_ => (DateTime.UtcNow - _shiftToBackgroundTime) > Interval.Value)
                .Subscribe(_ => StartNewSession())
                .AddTo(_disposables);
        }

        private void StartNewSession()
        {
            _prefs.SessionId.Value = Guid.NewGuid().ToString();
            _prefs.VersionSessionCounter++;
            _prefs.GlobalSessionCounter.Value++;
        }

        void IDisposable.Dispose()
        {
            _disposables?.Dispose();
        }
    }
}