﻿#if UNITY_EDITOR
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using Newtonsoft.Json;
using UnityEngine;

namespace Magify
{
    public partial class BinaryStorage
    {
        private static readonly HashSet<string> _lockedFiles = new();
        private static readonly Dictionary<string, object> _dictionary = new();
        private static readonly Dictionary<Type, PropertyInfo> _typeToDataGetter = new();

        static partial void SaveAsJsonForDebug(string filePath, List<Section> sections)
        {
#if MAGIFY_VERBOSE_LOGGING
            try
            {
                _dictionary.Clear();
                foreach (var section in sections.Where(c => c.HasData))
                {
                    var sectionType = section.GetType();
                    if (!_typeToDataGetter.TryGetValue(sectionType, out var getter))
                    {
                        getter = section.GetType().GetProperty("Data", BindingFlags.Instance | BindingFlags.NonPublic);
                        _typeToDataGetter[sectionType] = getter;
                    }
                    if (getter == null)
                    {
                        continue;
                    }
                    _dictionary[section.Name] = getter.GetMethod.Invoke(section, Array.Empty<object>());
                }
                File.WriteAllText(filePath + ".json", JsonConvert.SerializeObject(_dictionary, Formatting.Indented));
            }
            catch
            {
                // ignored
            }
#endif
        }

        static partial void ThrowIfFilePathLocked(string filePath)
        {
            filePath = Path.GetFullPath(filePath).TrimEnd('\\');
            if (_lockedFiles.Any(s => filePath == s))
            {
                throw new MagifyBinaryStorageLockedException(filePath);
            }
        }

        static partial void LockFilePathInEditor(string filePath)
        {
            filePath = Path.GetFullPath(filePath).TrimEnd('\\');
            _lockedFiles.Add(filePath);
        }

        static partial void UnlockFilePathInEditor(string filePath)
        {
            filePath = Path.GetFullPath(filePath).TrimEnd('\\');
            _lockedFiles.Remove(filePath);
        }
    }
}

#endif